@@ -1,10 +1,8 @@
-# Paul Glenn
-# curves.py
 # Give curve pairs class structure in preparation for public access
 import numpy as np
-from itertools import product
+from itertools import product, izip
 from copy import deepcopy
-
+import re
 
 def fix_matrix_signs(M):
     '''
@@ -547,6 +545,9 @@ def edges(M):
                     faces += 1
                     found = 1
                     all_faces.append((edges,face))
+    #from sys import stderr
+    #stderr.write('all_faces '+str(all_faces)+'\n')
+    #stderr.write('Paths '+str(Paths))
     return all_faces, Paths
 
 def boundary_count(M):
@@ -947,9 +948,112 @@ def test_perms(original_ladder):
             ladder_to_perm[0].append(first_vertex)
     return distance3, distance4
 
+#4+2-3-5-1+
+
+#[1,2,3,2,4]
+#[5,3,4,1,5]
+
+#1-6+4-2+5-7+3+
+
+#[7,4,7,2,4,2,6]
+#[1,3,6,3,5,1,5]
+
+#2+5-7+3+1-6+4-
+
+#[4,1,4,6,1,6,3]
+#[5,7,3,7,2,5,2]
+def ladder_to_cycle(ladder_top, ladder_bottom):
+    locations = {arc: {'top': [], 'bottom': []} for arc in set(ladder_top + ladder_bottom)}
+    for loc, varc in enumerate(ladder_top):
+        locations[varc]['top'].append(loc)
+
+    for loc, varc in enumerate(ladder_bottom):
+        locations[varc]['bottom'].append(loc)
+
+    n = len(ladder_top)
+    cycle = ''
+    #arbitrarily orient them positively
+    orientation = '+'
+    current = 'top'
+    start = 1
+    prev_loc = (ladder_top+ladder_bottom).index(start) % n
+    for i in range(n):
+        #get the top and bottom of the current vertex
+        top = locations[start]['top']
+        bottom = locations[start]['bottom']
+
+        # if there is an endpoint on both the top and bottom,
+        # then orientation is preserved and the location is simply
+        # whatever hasn't already been used. We switch side of the
+        # ladder accordingly
+        if top and bottom:
+            # switch side of ladder
+            current = 'top' if current == 'bottom' else 'bottom'
+            # preserve orientation
+            orientation = '-' if orientation == '-' else '+'
+            # get the locations avaiable there
+            locs = locations[start][current]
+            cycle += str(locs[0]+1) + orientation
+            if current == 'bottom':
+                start = ladder_top[locs[0]]
+            if current == 'top':
+                start = ladder_bottom[locs[0]]
+            prev_loc = locs[0]
+            current = 'top' if current == 'bottom' else 'bottom'
+        elif not top:
+            current = 'bottom'
+            orientation = '-' if orientation == '+' else '+'
+            locs = locations[start][current]
+            t = set(locs)
+            t.remove(prev_loc)
+            current_loc = t.pop()
+            cycle += str(current_loc+1)+orientation
+            start = ladder_top[current_loc]
+            prev_loc = current_loc
+            current = 'top'
+        elif not bottom:
+            current = 'top'
+            orientation = '-' if orientation == '+' else '+'
+            locs = locations[start][current]
+            t = set(locs)
+            t.remove(prev_loc)
+            current_loc = t.pop()
+            cycle += str(current_loc+1)+orientation
+            start = ladder_bottom[current_loc]
+            prev_loc = current_loc
+
+            current = 'bottom'
+    return cycle
 
 
+def cycle_to_ladder(cycle_rep):
+    arcs = [int(i) for i in re.split('[-+]', cycle_rep)[:-1]]
+    n = len(arcs)
+    signs = re.split('[0-9]+', cycle_rep)[1:]
+    top = [0 for i in range(len(arcs))]
+    bottom = [0 for i in range(len(arcs))]
+    ladder = [top, bottom]
+    
+    ladder_index = 0
+    for i in range(1, len(arcs)+1):
+        current_sign = signs.pop(0)
+        current_v = arcs.pop(0)
+        if current_sign == '+':
+            ladder[0][current_v-1] = i 
+            if i == 1:
+                ladder[1][current_v-1] = n#((i - 2) % n)
+            else:
+                ladder[1][current_v-1] = ((i - 1) % n)
 
+        if current_sign == '-':
+            ladder[1][current_v-1] = i 
+            if i == 1:
+                ladder[0][current_v-1] = n#((i - 2) % n)
+            else:
+                ladder[0][current_v-1] = ((i - 1) % n)
+
+
+    return ladder
 
 
 #import numpy as np
@@ -1004,23 +1108,34 @@ class CurvePair:
         self.boundaries = boundary_count(self.matrix)
         self.genus = genus(self.matrix)
         self.edges = edges(self.matrix)
-
+        #self.arc_boundary = self.edges[1]
         self.solution = vector_solution(self.edges[0])
 
         self.loops = []
+        self.dist = dist
+        self.conjectured_dist = conjectured_dist
+        self.computed_distance = False
+        self.recursive = recursive
+    
+    @property
+    def distance(self):
+        if self.computed_distance == False:
+            if self.dist is 1:
+                graph = Graph(self.edges, rep_num=self.conjectured_dist-2)
+                graph.compute_loops(self.n, self.genus)
+                self.loops = graph.gammas
+                #stderr.write(str(self.loops)+'\n')
+                self.computed_distance, self.loop_matrices = self.compute_distance(self.matrix, self.loops, recursive=self.recursive)
 
-        if dist is 1:
-            graph = Graph(self.edges, rep_num=conjectured_dist-2)
-            graph.compute_loops(self.n, self.genus)
-            self.loops = graph.gammas
-            #from sys import stderr
-            #stderr.write(str(self.loops)+'\n')
-            self.distance, self.loop_matrices = self.compute_distance(self.matrix, self.loops, recursive=recursive)
+            else:
+                self.computed_distance = None
+            return self.computed_distance
         else:
-            self.distance = None
+            return self.computed_distance
+
 
     def __repr__(self):
-        return self.ladder[0]+'\n'+self.ladder[1]+'\n'
+        return str(self.ladder[0])+'\n'+str(self.ladder[1])+'\n'
 
     def compute_distance(self, M, all_paths,recursive=True):
         '''
